@@ -2,8 +2,8 @@
 #' 
 #' Match one scene to another based on linear regression of pseudo-invariant features (PIF).
 #' 
-#' @param img RasterStack or RasterBrick. Image to be adjusted.
-#' @param ref RasterStack or RasterBrick. Reference image.
+#' @param img RasterStack or RasterBrick or SpatRaster. Image to be adjusted.
+#' @param ref RasterStack or RasterBrick or SpatRaster. Reference image.
 #' @param method Method to calculate pixel similarity. Options: euclidean distance ('ed'), spectral angle ('sam') or pearson correlation coefficient ('cor').
 #' @param quantile Numeric. Threshold quantile used to identify PIFs
 #' @param returnPifMap Logical. Return a binary raster map ot pixels which were identified as pesudo-invariant features.
@@ -22,7 +22,7 @@
 #' Model fitting is performed with simple linear models (\code{\link[stats]{lm}}); fitting one model per layer. 
 #' @return 
 #' Returns a List with the adjusted image and intermediate products (if requested). 
-#' #' \itemize{
+#' \itemize{
 #'    \item \code{img}: the adjusted image
 #'    \item \code{simMap}: pixel-wise similarity map (if \code{returnSimMap = TRUE})
 #'    \item \code{pifMap}: binary map of pixels selected as pseudo-invariant features (if \code{returnPifMap = TRUE}) 
@@ -30,17 +30,15 @@
 #' }
 #' @export 
 #' @examples 
-#' library(raster)
-#' 
-#' ## Import Landsat example data
-#' data(lsat)
+#' library(terra)
+#'
 #' 
 #' ## Create fake example data
 #' ## In practice this would be an image from another acquisition date
-#' lsat_b <- log(lsat)  
+#' lsat_b <- log(lsat)
 #' 
 #' ## Run pifMatch and return similarity layer, invariant features mask and models
-#' lsat_b_adj <- pifMatch(lsat_b, lsat, returnPifMap = TRUE, 
+#' lsat_b_adj <- pifMatch(lsat_b, lsat, returnPifMap = TRUE,
 #'                          returnSimMap = TRUE, returnModels = TRUE)
 #' \donttest{
 #' ## Pixelwise similarity
@@ -59,46 +57,55 @@
 #' summary(lsat_b_adj$models[[1]])
 #' }
 pifMatch <- function(img, ref, method = "cor", quantile = 0.95, returnPifMap = TRUE, returnSimMap = TRUE, returnModels = FALSE){
-    
-	img <- .toRaster(img)
-	ref <- .toRaster(ref)
-	
-	if(nlayers(img)!=nlayers(ref) | nlayers(img) <= 1) stop("Both images need at least two corresponding bands and must have the same number of bands.", call.=FALSE)
-    
+    img <- .toTerra(img)
+	ref <- .toTerra(ref)
+
+     if(nlyr(img)!=nlyr(ref) | nlyr(img) <= 1)
+      stop("Both images need at least two corresponding bands and must have the same number of bands.", call.=FALSE)
+
     imgfull <- img
     ## Get joint extent
-    if(!extent(img)==extent(ref)) { 
+    if(!ext(img)==ext(ref)) {
         img <- crop(img, ref)
         ref <- crop(ref, img)
     }
     
     ## Calculate pixelwise similarity
-    if(!method %in% c("ed", "sam", "cor")) stop("method must be one of 'ed', 'cor' or 'sam'", call. = FALSE)
-    nmeth <- c(ed=1, sam=2, cor=3)[method]    
-    pifield <- overlay(img, ref, fun = function(x,y) {pwSimilarityCpp(x,y,nmeth)})
+    if(!method %in% c("ed", "sam", "cor"))
+      stop("method must be one of 'ed', 'cor' or 'sam'", call. = FALSE)
+
+    nmeth <- c(ed=1, sam=2, cor=3)[method]
+
+    pifield_values <- pwSimilarityCpp(img[], ref[] ,nmeth)
+    pifield <- img[[1]]
+    values(pifield) <- pifield_values
+    names(pifield) <- "layer"
+
     names(pifield) <- method
+
     ## Similarity quantile threshold
-    thresh  <- quantile(pifield, p = quantile)
+    thresh  <- quantile(values(pifield), p = quantile)
     pi      <- pifield > thresh
+
     names(pi) <- "pifMap"
-    
+
     ## Fit linear models
-    models <- lapply(1:nlayers(img), function(i){
-                df <- data.frame(img = img[[i]][pi], ref = ref[[i]][pi])
-                mod <- lm(ref~img, data = df)
-            })
-    
+    models <- lapply(1:nlyr(img), function(i){
+        df <- data.frame(img = img[[i]][pi], ref = ref[[i]][pi])
+        colnames(df) <- c("img", "ref")
+        mod <- lm(ref ~ img, data = df)
+    })
+
     ## Predict linear models for whole raster
     correct <- lapply(seq_along(models), function(i){
-                corLayer <- imgfull[[i]]
-                mod <- models[[i]]
-                names(corLayer) <- "img"
-                predict(corLayer, mod)
-            })
+        corLayer <- imgfull[[i]]
+        mod <- models[[i]]
+        names(corLayer) <- "img"
+        predict(corLayer, mod)
+    })
     
     ## Assemble and return output
     names(models) <- names(img)
-    correct <- stack(correct)
     names(correct) <- names(imgfull)
     
     out <- list( img = correct, simMap = pifield, pifMap = pi, models = models )
@@ -109,10 +116,4 @@ pifMatch <- function(img, ref, method = "cor", quantile = 0.95, returnPifMap = T
     )
     out[!names(out) %in% ret] <- NULL
     return(out)
-} 
-
-
-
-
-
-
+}
