@@ -1,9 +1,7 @@
 context("getValidation")
 suppressPackageStartupMessages(library("randomForest"))
-suppressPackageStartupMessages(library("sp"))
 
 train <- readRDS(system.file("external/trainingPoints.rds", package="RStoolbox"))
-data(rlogo)
 train$num <- rnorm(nrow(train))
 class   <- superClass(rlogo, trainData = train, responseCol = "class", tuneLength = 1, trainPartition = 0.7, predict = FALSE)
 reg     <- superClass(rlogo, trainData = train, responseCol = "num", tuneLength = 1, trainPartition = 0.7, predict = FALSE, mode = "regression")
